@@ -17,22 +17,39 @@
 package kafka.security.auth
 
 import kafka.common.KafkaException
-import org.junit.{Test, Assert}
-import org.scalatest.junit.JUnitSuite
+import org.junit.Assert.assertEquals
+import org.junit.Test
+import org.scalatest.Assertions.fail
+import org.apache.kafka.common.resource.{ResourceType => JResourceType}
 
-class ResourceTypeTest extends JUnitSuite {
+@deprecated("Scala Authorizer API classes gave been deprecated", "Since 2.5")
+class ResourceTypeTest {
 
   @Test
   def testFromString(): Unit = {
     val resourceType = ResourceType.fromString("Topic")
-    Assert.assertEquals(Topic, resourceType)
+    assertEquals(Topic, resourceType)
 
     try {
       ResourceType.fromString("badName")
       fail("Expected exception on invalid ResourceType name.")
     } catch {
-      case e: KafkaException => // expected
+      case _: KafkaException => // expected
     }
   }
 
+  /**
+    * Test round trip conversions between org.apache.kafka.common.acl.ResourceType and
+    * kafka.security.auth.ResourceType.
+    */
+  @Test
+  def testJavaConversions(): Unit = {
+    JResourceType.values.foreach {
+      case JResourceType.UNKNOWN | JResourceType.ANY =>
+      case jResourceType =>
+        val resourceType = ResourceType.fromJava(jResourceType)
+        val jResourceType2 = resourceType.toJava
+        assertEquals(jResourceType, jResourceType2)
+    }
+  }
 }
